from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.audio_models import CreateAudioInput, CreateAudioOutput

__all__ = ["CreateAudioInput", "CreateAudioOutput", "create_audio"]


def create_audio(
    *,
    conversation: list[tuple[str, str]] = DEFAULT,
    pause_duration: float = DEFAULT,
    voice: dict[str, str] = DEFAULT,
) -> CreateAudioOutput:
    """
    Takes a list of conversation turns with specified voices and generates an audio file
    with appropriate pauses between turns.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = CreateAudioInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="audio", name="create_audio", input=input_model.model_dump()
    )
    return CreateAudioOutput.model_validate(output_model)



import typing as t

from pydantic import BaseModel, Field

from common_py.model import enums

from . import ToolOutput


class PageOptions(BaseModel):
    """Options for configuring page scraping behavior."""

    headers: dict | None = None
    include_html: bool = False  # This will need special handling to map to formats
    only_main_content: bool = True
    exclude_tags: list[str] | None = None
    wait_for: int = 0


class ExtractorOptions(BaseModel):
    """Options for configuring content extraction behavior."""

    extraction_prompt: str | None = None  # Maps to jsonOptions.prompt
    extraction_schema: dict | None = None  # Maps to jsonOptions.schema


class ScrapeURLInput(BaseModel):
    url: str = Field(
        default="",
        json_schema_extra={
            "mandatory": True,
            "field_type": enums.CustomDataType.URL.value,
        },
    )
    page_options: PageOptions = Field(
        default_factory=PageOptions, description="The options for scraping the page"
    )
    extractor_options: ExtractorOptions = Field(
        default_factory=ExtractorOptions,
        description="The options for extracting content via service LLM",
    )
    timeout: int = 60000


class ScrapeURLOutput(ToolOutput):
    content: dict[str, t.Any]


class CrawlURLInput(BaseModel):
    url: str = Field(
        default="",
        json_schema_extra={
            "mandatory": True,
            "field_type": enums.CustomDataType.URL.value,
        },
    )
    limit: int = Field(default=10, description="Maximum number of pages to crawl")
    max_depth: int = Field(default=2, description="Maximum depth for the crawl")
    ignore_sitemap: bool = Field(
        default=True, description="Whether to ignore the sitemap when crawling"
    )
    page_options: PageOptions = Field(
        default_factory=PageOptions, description="The options for scraping each page"
    )
    extractor_options: ExtractorOptions = Field(
        default_factory=ExtractorOptions,
        description="The options for extracting content via service LLM",
    )
    timeout: int = 600


class CrawlURLOutput(ToolOutput):
    data: list[dict[str, t.Any]] = Field(
        default_factory=list, description="List of scraped page data from the crawl"
    )
    status: str = Field(default="", description="Status of the crawl operation")
    job_id: str | None = Field(
        default=None, description="Job ID if crawl was processed asynchronously"
    )
    credits_used: int | None = Field(
        default=None, description="Number of credits used for the crawl"
    )


class MapURLInput(BaseModel):
    """Input parameters for the Firecrawl Map endpoint."""

    url: str = Field(
        default="",
        json_schema_extra={
            "mandatory": True,
            "field_type": enums.CustomDataType.URL.value,
        },
    )
    search: str | None = None
    ignore_sitemap: bool = True
    sitemap_only: bool = False
    include_subdomains: bool = False
    limit: int = 5000
    timeout: int = 120000


class MapURLOutput(ToolOutput):
    """Output from the Firecrawl Map endpoint."""

    success: bool = False
    links: list[str] = Field(default_factory=list)



import os
import typing as t

from pydantic import BaseModel, Field

from common_py.model.core import CSVDataResponse

from . import ToolOutput


class SnowflakeConfig(BaseModel):
    account_name: str | None = Field(
        default=os.getenv("SNOWFLAKE_ACCOUNT", ""), description="Snowflake account name"
    )
    username: str | None = Field(
        default=os.getenv("SNOWFLAKE_USERNAME", ""), description="Snowflake username"
    )
    password: str | None = Field(
        default=os.getenv("SNOWFLAKE_PASSWORD", ""), description="Snowflake password"
    )
    warehouse: str | None = Field(
        default=os.getenv("SNOWFLAKE_WAREHOUSE", ""), description="Snowflake warehouse"
    )
    database: str | None = Field(
        default=os.getenv("SNOWFLAKE_DATABASE", ""), description="Snowflake database"
    )
    role: str | None = Field(
        default=os.getenv("SNOWFLAKE_ROLE", ""), description="Snowflake role"
    )
    schema_name: str | None = Field(
        default=os.getenv("SNOWFLAKE_SCHEMA", ""), description="Snowflake schema"
    )


class UploadDataInput(BaseModel):
    config: SnowflakeConfig = Field(
        description="Snowflake connection configuration", default_factory=SnowflakeConfig
    )
    dataframe: t.Any = Field(
        description="DataFrame or dict representation of DataFrame to upload"
    )
    table_name: str = Field(description="Name of the table to create/update")
    if_exists: t.Literal["fail", "replace", "append"] = Field(
        "replace", description="How to behave if the table already exists"
    )


class TableInfo(BaseModel):
    """Information about a Snowflake table."""

    table_name: str
    database: str | None
    schema_name: str | None
    rows_affected: int


class UploadDataOutput(ToolOutput):
    table_info: TableInfo


class ReadTableInput(BaseModel):
    """Input for reading a table from Snowflake."""

    config: SnowflakeConfig = Field(description="Snowflake connection configuration")
    table_name: str = Field(description="Name of the table to read")
    database: str | None = Field(
        None, description="Database containing the table (defaults to connection config)"
    )
    schema_name: str | None = Field(
        None, description="Schema containing the table (defaults to connection config)"
    )
    limit: int | None = Field(
        None, description="Maximum number of rows to read (None means all rows)"
    )


class ReadTableOutput(ToolOutput):
    table_info: TableInfo
    data: CSVDataResponse


class ExecuteQueryInput(BaseModel):
    """Input for executing a SQL query in Snowflake."""

    config: SnowflakeConfig = Field(description="Snowflake connection configuration")
    query: str = Field(..., description="SQL query to execute")
    page: int = Field(1, description="Page number for paginated results (1-indexed)")
    page_size: int = Field(100, description="Number of rows per page")


class ExecuteQueryOutput(ToolOutput):
    data: CSVDataResponse



from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.pii_models import RedactInput, RedactOutput

__all__ = ["RedactInput", "RedactOutput", "redact"]


def redact(*, text: str = DEFAULT) -> RedactOutput:
    """
    Redact PII (Personally Identifiable Information) from input text using LLM.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = RedactInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="pii", name="redact", input=input_model.model_dump()
    )
    return RedactOutput.model_validate(output_model)



from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.call_task_generator_models import (
    CallTaskDefinition,
    CallTaskGeneratorInput,
    CallTaskGeneratorOutput,
)

__all__ = [
    "CallTaskDefinition",
    "CallTaskGeneratorInput",
    "CallTaskGeneratorOutput",
    "generate_call_task",
]


def generate_call_task(
    *,
    task_name: str = DEFAULT,
    current_instructions: str = DEFAULT,
    current_call_task_definition: CallTaskDefinition | None = DEFAULT,
    new_instructions: str = DEFAULT,
) -> CallTaskGeneratorOutput:
    """
    Generate a call task using LLM to create or update a phone call task definition.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = CallTaskGeneratorInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="call_task_generator",
        name="generate_call_task",
        input=input_model.model_dump(),
    )
    return CallTaskGeneratorOutput.model_validate(output_model)



from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.edgartool_models import (
    GetLatestCompanyFilingTextInput,
    GetLatestCompanyFilingTextOutput,
)

__all__ = [
    "GetLatestCompanyFilingTextInput",
    "GetLatestCompanyFilingTextOutput",
    "get_latest_company_filing_text",
]


def get_latest_company_filing_text(
    *, company_identifier: str, filing_types: list[str]
) -> GetLatestCompanyFilingTextOutput:
    """Get the latest company filing text by company identifier and filing types
    using Edgar API."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetLatestCompanyFilingTextInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="edgartool",
        name="get_latest_company_filing_text",
        input=input_model.model_dump(),
    )
    return GetLatestCompanyFilingTextOutput.model_validate(output_model)



from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.react_agent_models import ReactAgentInput, ReactAgentOutput

__all__ = ["ReactAgentInput", "ReactAgentOutput", "react_agent"]


def react_agent(
    *,
    task: str,
    max_iterations: int = DEFAULT,
    context: str = DEFAULT,
    available_tools: list[str] = DEFAULT,
    model_name: str = DEFAULT,
    system_message: str = DEFAULT,
    num_examples: int = DEFAULT,
) -> ReactAgentOutput:
    """
    Execute a React agent to solve a task using code and tool execution.

    This tool utilizes the PythonReActAgent to solve tasks by thinking, running code,
    observing results, and iterating until it finds an answer.
    This is used when you want to put a react tool within a node
    if you want to just create the workflow with react there is a different way to do it
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ReactAgentInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="react_agent", name="react_agent", input=input_model.model_dump()
    )
    return ReactAgentOutput.model_validate(output_model)



from typing import Any

from pydantic import BaseModel, Field

from . import ToolOutput


class CallTaskDefinition(BaseModel):
    """Definition of a task for voice calls"""

    task_name: str
    instructions: str
    input_schema: dict[str, Any]
    output_schema: dict[str, Any]


class CallTaskGeneratorInput(BaseModel):
    task_name: str = Field(
        json_schema_extra={"mandatory": True},
        description="The name of the task to generate a call script for",
        default="",
    )
    current_instructions: str = Field(
        json_schema_extra={"mandatory": True},
        description="The current instructions for the call task",
        default="",
    )
    current_call_task_definition: CallTaskDefinition | None = Field(
        json_schema_extra={"mandatory": True},
        description="The current definition of the call task",
        default=None,
    )
    new_instructions: str = Field(
        json_schema_extra={"mandatory": True},
        description="The new instructions on how to improve the call task definition",
        default="",
    )


class CallTaskGeneratorOutput(ToolOutput):
    task_name: str = Field(
        json_schema_extra={"mandatory": True},
        description="The name of the task to generate a call script for",
        default="",
    )
    call_task_definition: CallTaskDefinition | None = Field(
        json_schema_extra={"mandatory": True},
        description="The call task definition",
        default=None,
    )



import typing as t

from pydantic import BaseModel, Field

from common_py.model import enums

from . import ToolOutput


class PodcastSection(BaseModel):
    """Model for a section of the podcast."""

    title: str
    key_points: list[str]
    estimated_turns: int


class CreatePodcastInput(BaseModel):
    model: str = Field(
        default=enums.ModelName.GPT_4O,
        description="The model to use for the podcast creation",
    )
    user_1_profile: str = Field(
        default="", description="Profile/description of the first speaker"
    )
    user_2_profile: str = Field(
        default="", description="Profile/description of the second speaker"
    )
    content: str = Field(
        default="", description="The content to be discussed in the podcast"
    )


class CreatePodcastOutput(ToolOutput):
    result: dict[str, t.Any]



import typing as t

from pydantic import BaseModel, Field, field_validator

from common_py.sql.model import SQADatabaseConfig

from . import ToolOutput


class SourceSelectionInput(BaseModel):
    """Input model for source/table selection"""

    query: str = Field(
        description="Natural language query to analyze for table selection"
    )
    num_tables: int = Field(
        default=3, ge=1, description="Number of most relevant tables to return"
    )
    database_config: SQADatabaseConfig = Field(
        description="Database configuration for connecting to the target database"
    )

    @classmethod
    @field_validator("database_config", mode="before")
    def validate_database_config(cls, v: t.Any) -> SQADatabaseConfig:
        """Transform dict to SQADatabaseConfig and validate configuration"""
        # If it's already an SQADatabaseConfig, use it directly
        if isinstance(v, SQADatabaseConfig):
            config = v
        # If it's a dict, transform to SQADatabaseConfig
        elif isinstance(v, dict):
            try:
                config = SQADatabaseConfig(**v)
            except Exception as e:
                raise ValueError(f"Invalid database configuration: {str(e)}")
        else:
            raise ValueError(
                "database_config must be a dictionary or SQADatabaseConfig object"
            )

        # Basic validation checks
        if not config.database:
            raise ValueError("Database name is required")

        if config.dialect.value not in [
            "postgresql",
            "mysql",
            "sqlite",
            "trino",
            "mssql",
            "oracle",
            "snowflake",
        ]:
            raise ValueError(f"Unsupported database dialect: {config.dialect.value}")

        # For non-SQLite databases, validate credentials
        if config.dialect.value != "sqlite":
            if not config.credentials:
                raise ValueError(
                    "Database credentials are required for non-SQLite databases"
                )
            if not config.credentials.host:
                raise ValueError("Database host is required")
            if not config.credentials.username:
                raise ValueError("Database username is required")
            if not config.credentials.password:
                raise ValueError("Database password is required")

        return config


class SourceSelectionOutput(ToolOutput):
    """Output model for source/table selection"""

    relevant_tables: list[dict[str, t.Any]] = Field(
        description="List of tables selected with relevance scores and reasoning"
    )
    explanation: str = Field(
        description="Overall explanation of the table selection process"
    )
    table_definitions: list[dict[str, t.Any]] = Field(
        description="Detailed definitions of the selected tables"
    )


class NLToSQLInput(BaseModel):
    """Input model for NL-to-SQL conversion"""

    query: str = Field(description="Natural language query to convert to SQL")
    num_tables_to_filter_for_sql_generation: int = Field(
        default=3,
        ge=1,
        description="Number of most relevant tables to select for SQL generation",
    )
    sample_rows: t.Optional[dict[str, list[dict[str, t.Any]]]] = Field(
        default=None,
        description="Optional sample data to provide context for SQL generation",
    )
    database_config: SQADatabaseConfig = Field(
        description="Database configuration for connecting to the target database"
    )

    @classmethod
    @field_validator("database_config", mode="before")
    def validate_database_config(cls, v: t.Any) -> SQADatabaseConfig:
        """Transform dict to SQADatabaseConfig and validate configuration"""
        # If it's already an SQADatabaseConfig, use it directly
        if isinstance(v, SQADatabaseConfig):
            config = v
        # If it's a dict, transform to SQADatabaseConfig
        elif isinstance(v, dict):
            try:
                config = SQADatabaseConfig(**v)
            except Exception as e:
                raise ValueError(f"Invalid database configuration: {str(e)}")
        else:
            raise ValueError(
                "database_config must be a dictionary or SQADatabaseConfig object"
            )

        # Basic validation checks
        if not config.database:
            raise ValueError("Database name is required")

        if config.dialect.value not in [
            "postgresql",
            "mysql",
            "sqlite",
            "trino",
            "mssql",
            "oracle",
            "snowflake",
        ]:
            raise ValueError(f"Unsupported database dialect: {config.dialect.value}")

        # For non-SQLite databases, validate credentials
        if config.dialect.value != "sqlite":
            if not config.credentials:
                raise ValueError(
                    "Database credentials are required for non-SQLite databases"
                )
            if not config.credentials.host:
                raise ValueError("Database host is required")
            if not config.credentials.username:
                raise ValueError("Database username is required")
            if not config.credentials.password:
                raise ValueError("Database password is required")

        return config


class NLToSQLOutput(ToolOutput):
    """Output model for NL-to-SQL conversion"""

    sql_query: str = Field(description="Generated PostgreSQL query")
    dialect_sql_query: t.Optional[str] = Field(
        default=None, description="Query converted to target database dialect"
    )
    explanation: str = Field(description="Explanation of the generated SQL query")
    table_selection_explanation: str = Field(
        description="Explanation of why specific tables were selected"
    )
    selected_tables: list[dict[str, t.Any]] = Field(
        description="List of tables that were selected for the query"
    )
    database_name: str = Field(description="Name of the database used")



from pydantic import BaseModel, Field

from common_py.model import enums

from . import ToolOutput


class ReactAgentInput(BaseModel):
    """Input model for the React agent tool."""

    task: str = Field(
        description="The task or question to be solved by the React agent",
        json_schema_extra={"mandatory": True},
    )
    max_iterations: int = Field(
        default=15, description="Maximum number of iterations before giving up"
    )
    context: str = Field(default="", description="Additional context for the React agent")
    available_tools: list[str] = Field(
        default=[], description="List of available tools for the agent to use"
    )
    model_name: str = Field(
        default=enums.ModelName.O4_MINI,
        description="The LLM model to use for the React agent",
    )
    system_message: str = Field(
        default="You are a Python coding assistant that solves problems step by step"
        " using code.",
        description="System message for the LLM",
    )
    num_examples: int = Field(
        default=2, description="Number of k-shot examples to include"
    )


class ReactAgentOutput(ToolOutput):
    """Output model for the React agent tool."""

    answer: str = Field(description="The final answer from the React agent")
    success: bool = Field(description="Whether the agent completed successfully")
    reasoning_trace: list[str] | None = Field(
        default=None, description="The full reasoning trace if requested"
    )



from typing import Any

from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.nl_to_sql_models import (
    NLToSQLInput,
    NLToSQLOutput,
    SourceSelectionInput,
    SourceSelectionOutput,
    SQADatabaseConfig,
)

__all__ = [
    "NLToSQLInput",
    "NLToSQLOutput",
    "SQADatabaseConfig",
    "SourceSelectionInput",
    "SourceSelectionOutput",
    "generate_sql",
]


def generate_sql(
    *,
    query: str,
    num_tables_to_filter_for_sql_generation: int = DEFAULT,
    sample_rows: dict[str, list[dict[str, Any]]] | None = DEFAULT,
    database_config: SQADatabaseConfig,
) -> NLToSQLOutput:
    """
    Convert natural language queries to SQL with automatic table selection.

    This tool combines table selection and SQL generation into a single step:
    1. Analyzes your query to select the most relevant tables
    2. Generates optimized SQL using the selected tables
    3. Converts to the appropriate SQL dialect for your database
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = NLToSQLInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="nl_to_sql", name="generate_sql", input=input_model.model_dump()
    )
    return NLToSQLOutput.model_validate(output_model)



from typing import Any

from pydantic import BaseModel, Field

from . import ToolOutput


class RunTasksInParallelInput(BaseModel):
    """Input for running tasks in parallel."""

    func_to_run: Any = Field(description="The function to execute in parallel")
    items_to_process: list[Any] = Field(
        description="List of items to process in parallel"
    )
    shared_args: list[Any] = Field(
        description="Additional arguments to pass to each function call"
    )
    max_workers: int = Field(default=5, description="Maximum number of workers to use")


class RunTasksInParallelOutput(ToolOutput):
    """Output from running tasks in parallel."""

    results: list[Any] = Field(description="List of results from parallel execution")



import typing as t

from pydantic import BaseModel, Field

from . import ToolOutput


class SearchCompaniesInput(BaseModel):
    """Input data for the PitchBook Tool."""

    data: dict = Field(default={}, description="Parameter to provide payload.")
    params: dict = Field(
        default={}, description="Parameter to provide GET parameters in URL."
    )
    timeout: float = Field(default=60, description="Request timeout in seconds")


class SearchCompaniesOutput(ToolOutput):
    """Output data from the PitchBook Tool."""

    content: t.Any = Field(
        default=None, description="Structured content from the operation"
    )


class GetCompanyInfoInput(BaseModel):
    """Input data for the PitchBook Tool."""

    data: dict = Field(default={}, description="Parameter to provide payload.")
    params: dict = Field(
        default={}, description="Parameter to provide GET parameters in URL."
    )
    timeout: float = Field(default=60, description="Request timeout in seconds")


class GetCompanyInfoOutput(ToolOutput):
    """Output data from the PitchBook Tool."""

    content: t.Any = Field(
        default=None, description="Structured content from the operation"
    )



import json
import socket
from datetime import datetime

from common_py.context import ContextWrapper


class WorkflowInitializer:
    def __init__(self):
        pass

    @staticmethod
    def initialize() -> None:
        # Import here to avoid circular dependency
        from external.a79.src.a79.client import A79Client

        api_client = A79Client()

        user = api_client.raw_request(method="GET", url="/api/v1/user/me", json={})

        # NOTE: The context is needed because we are executing nodes via library
        # calls which require a context while making db updates.
        # This will be reworked once we move to e2b and run the nodes via the API.
        ContextWrapper.init_from_json(
            json.dumps({"org_id": user["org_id"], "user_id": user["auth0_user_id"]})
        )

        # Generate timestamp string in format: YYYYMMDD_HHMMSS
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        hostname = socket.gethostname()

        base_name = f"sdk-run-{hostname}-{timestamp}"

        folder_response = api_client.raw_request(
            method="POST", url="/api/v1/folder", json={"name": base_name}
        )
        print(folder_response, flush=True)



from enum import Enum


class RunStatus(str, Enum):
    # NOTE: When new statuses are added, alembic does not pick up the new enum
    # values. So, we need to create an alembic migration to include the
    # new statuses.
    UNDEFINED = "undefined"
    NOT_STARTED = "not_started"
    PENDING = "pending"
    RUNNING = "running"
    FAILED = "failed"
    SUCCEEDED = "succeeded"
    PARTIAL_SUCCESS = "partial_success"
    SKIPPED = "skipped"
    PAUSED = "paused"
    CANCELLED = "cancelled"



import typing as t

from pydantic import BaseModel, ConfigDict, Field, field_serializer, field_validator
from pydantic.json_schema import SkipJsonSchema

from common_py.model import enums
from common_py.workflow.nodes.node import ErrorHandling
from common_py.workflow.nodes.workflow_types import InferenceMode
from common_py.workflow.utils.pydantic_model_serializer import (
    deserialize_model,
    serialize_model,
)

from . import ToolOutput


class LLMUsageData(BaseModel):
    prompt_tokens: int
    prompt_tokens_cost_usd: float | None
    completion_tokens: int
    completion_tokens_cost_usd: float | None
    total_tokens: int
    total_tokens_cost_usd: float | None


class ChatInput(BaseModel):
    MODEL_PREFIX: t.ClassVar[str | None] = None

    schema: dict[str, t.Any] | type[BaseModel] | None = None  # type: ignore
    model: str = enums.ModelName.GPT_4O_MINI
    prompt_str: str | None = None
    temperature: float = 0.1
    stop: list[str] | None = None
    error_handling: ErrorHandling = ErrorHandling(timeout_seconds=600)
    top_p: float | None = None
    seed: int | None = None
    presence_penalty: float | None = None
    frequency_penalty: float | None = None
    tool_choice: str | None = None
    inference_mode: InferenceMode = InferenceMode.DEFAULT
    output_schema: dict[str, t.Any] | type[BaseModel] | None = Field(
        default=None, description="Schema for structured output or function calling."
    )
    stream: bool = Field(default=False, description="Enable streaming response")

    model_config = ConfigDict(arbitrary_types_allowed=True, extra="allow")

    @field_serializer("output_schema")
    def serialize_schema(self, value: t.Any | None, _info) -> dict[str, t.Any] | None:  # type: ignore
        if value is None:
            return None  # type: ignore
        elif isinstance(value, type) and issubclass(value, BaseModel):
            return serialize_model(value)
        else:
            return value  # type: ignore

    @field_validator("model")
    @classmethod
    def set_model(cls, value: str | None) -> str:
        """Set the model with the appropriate prefix.

        Args:
            value (str | None): The model value.

        Returns:
            str: The model value with the prefix.
        """
        if cls.MODEL_PREFIX is not None and not value.startswith(cls.MODEL_PREFIX):  # type: ignore
            value = f"{cls.MODEL_PREFIX}{value}"
        return value  # type: ignore

    @field_validator("output_schema")
    @classmethod
    def parse_inference_schema(
        cls, value: dict[str, t.Any] | type[BaseModel] | None
    ) -> dict[str, t.Any] | type[BaseModel] | None:
        if value is None:
            return None
        elif isinstance(value, dict):
            try:
                return deserialize_model(value)
            except Exception:
                return value
        else:
            return value


class ChatOutput(ToolOutput):
    llm_call: SkipJsonSchema[dict[str, t.Any]] = Field(default_factory=dict)
    content: str | None = Field(default="")
    schema_result_json: dict[str, t.Any] | None = Field(default=None)
    tool_calls: SkipJsonSchema[list[dict[str, t.Any]] | None] = Field(default=None)



import typing as t

from pydantic import BaseModel, Field

from common_py.workflow.nodes.node import HumanReadableNodeOutput

DEFAULT: t.Any = object()
"""
Sentinel value for when a field has not been provided as input.
"""


class ToolSummary(BaseModel):
    short_summary: str
    long_summary: HumanReadableNodeOutput = Field(default_factory=HumanReadableNodeOutput)


class ToolOutput(BaseModel):
    tool_summary: ToolSummary



from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.podcast_models import (
    CreatePodcastInput,
    CreatePodcastOutput,
    PodcastSection,
)

__all__ = [
    "CreatePodcastInput",
    "CreatePodcastOutput",
    "PodcastSection",
    "create_podcast",
]


def create_podcast(
    *,
    model: str = DEFAULT,
    user_1_profile: str = DEFAULT,
    user_2_profile: str = DEFAULT,
    content: str = DEFAULT,
) -> CreatePodcastOutput:
    """Execute the podcast creation with the provided input data."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = CreatePodcastInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="podcast", name="create_podcast", input=input_model.model_dump()
    )
    return CreatePodcastOutput.model_validate(output_model)



import pandas as pd


def get_thoughtspot_data_type(dtype):
    if dtype == "int":
        return "INT64"
    elif dtype in ["float", "float64", "double"]:
        return "FLOAT"
    elif dtype in ["date", "datetime64[ns]", "datetime"]:
        return "DATE"
    else:
        return "VARCHAR"


# Helper function to convert cells to DataFrame
def excel_cells_to_dataframe(cells: list[dict]) -> pd.DataFrame:
    # Extract cells from the content
    max_row = max(cell["row"] for cell in cells)
    max_col = max(cell["column"] for cell in cells)

    # Create an empty DataFrame with appropriate dimensions
    df = pd.DataFrame(index=range(1, max_row + 1), columns=range(1, max_col + 1))

    # Fill the DataFrame with values
    for cell in cells:
        row_idx = cell["row"]
        col_idx = cell["column"]

        # Extract the value based on the data_type
        cell_value = cell["value"]["value"]
        data_type = cell["value"]["data_type"]

        # Convert to the appropriate Python type
        if data_type == "int":
            cell_value = int(cell_value) if cell_value is not None else None
        elif data_type == "float":
            cell_value = float(cell_value) if cell_value is not None else None
        elif data_type == "bool":
            cell_value = bool(cell_value) if cell_value is not None else None
        elif data_type == "datetime":
            cell_value = (
                pd.to_datetime(cell_value, unit="s") if cell_value is not None else None
            )
        # Add more type conversions as needed

        df.at[row_idx, col_idx] = cell_value

    # Rename columns to better match typical DataFrame structure (optional)
    df.columns = [f"col_{i}" for i in df.columns]

    return df


def convert_celldata_to_dataframe_filtered(cell_data_list: list[dict]) -> pd.DataFrame:
    # First create the complete DataFrame
    df_full = excel_cells_to_dataframe(cell_data_list)

    # Filter out rows and columns that are entirely NaN
    # Drop rows where all values are NaN
    df_filtered = df_full.dropna(how="all")

    # Drop columns where all values are NaN
    df_filtered = df_filtered.dropna(axis=1, how="all")

    # Reset the index for cleaner output
    df_filtered = df_filtered.reset_index(drop=True)

    return df_filtered


def transform_wide_to_long(df):
    # Create a copy of the dataframe to avoid modifying the original
    df_copy = df.copy()

    # Find the indices of rows containing the metric names
    # Assuming these are the rows with string values in the first column
    metric_rows = (
        df_copy.iloc[:, 0].apply(lambda x: isinstance(x, str) and x != "NaN").values
    )
    print("metric_rows:", metric_rows, flush=True)

    # Find which row contains the dates (typically the first row)
    date_row_idx = 0

    # Exclude the date row from metric rows
    metric_rows[date_row_idx] = False

    # Extract the metric names from the first column (excluding the date row)
    metric_names = df_copy.iloc[metric_rows, 0].tolist()
    print("metric_names:", metric_names, flush=True)

    # Extract dates from the date row (all columns except the first)
    date_columns = df_copy.columns[1:]  # Skip the first column as it contains labels
    dates = df_copy.loc[date_row_idx, date_columns].tolist()

    # Initialize lists to store the transformed data
    transformed_data = []

    # For each date column, extract the metric values
    for i, date_col in enumerate(date_columns):
        date = dates[i]
        row_data = {"date": date}

        # Extract values for each metric at this date
        for metric_idx, metric_name in zip(df_copy.index[metric_rows], metric_names):
            row_data[metric_name] = df_copy.loc[metric_idx, date_col]

        transformed_data.append(row_data)

    # Create the new dataframe
    new_df = pd.DataFrame(transformed_data)

    # Convert date strings to datetime objects if they're strings
    if new_df["date"].dtype == "object":
        new_df["date"] = pd.to_datetime(new_df["date"], errors="coerce", unit="s")

    # Convert numeric columns to appropriate types
    for col in new_df.columns:
        if col != "date":
            new_df[col] = pd.to_numeric(new_df[col], errors="coerce")

    return new_df

    # Helper function to parse cell range


def parse_cell_range(cell_range_str: str) -> tuple[int, int, int, int]:
    import re

    # Parse A1:C4 format
    match = re.match(r"([A-Z]+)(\d+):([A-Z]+)(\d+)", cell_range_str)
    if not match:
        raise ValueError(f"Invalid cell range format: {cell_range_str}")

    start_col_letter, start_row_str, end_col_letter, end_row_str = match.groups()

    # Convert column letters to numbers (A=1, B=2, etc.)
    def col_letter_to_number(col_letter):
        num = 0
        for c in col_letter:
            num = num * 26 + (ord(c) - ord("A") + 1)
        return num

    start_col = col_letter_to_number(start_col_letter)
    end_col = col_letter_to_number(end_col_letter)
    start_row = int(start_row_str)
    end_row = int(end_row_str)

    return start_col, start_row, end_col, end_row



from typing import Any, Optional

from common_py.model.retention_analysis_models import (
    ChartConfigData,
    RetentionChartingInputData,
    ToggleConfiguration,
    ToggleInfo,
    ToggleOption,
    ToggleSection,
    ToggleSelection,
)

__all__ = [
    "ChartConfigData",
    "RetentionChartingInputData",
    "ToggleConfiguration",
    "ToggleInfo",
    "ToggleOption",
    "ToggleSection",
    "ToggleSelection",
    "parse_control_panel_sheet",
    "get_toggle_configurations",
    "get_toggle_edit_operations",
]


def parse_control_panel_sheet(sheet_data: dict) -> list[ToggleInfo]:
    """
    Parse the control panel sheet data to extract toggle configurations.

    Args:
        sheet_data: Dictionary containing sheet data with cells array

    Returns:
        List of ToggleInfo objects representing each toggle section
    """
    toggles = []
    current_section: Optional[ToggleSection] = None
    current_options: list[ToggleOption] = []
    current_cell: Optional[str] = None

    # Create a dictionary for faster cell lookup by row/column
    cell_map = {}
    for cell in sheet_data["cells"]:
        if cell["value"] is None:
            continue
        cell_map[(cell["row"], cell["column"])] = cell["value"]["value"]

    # Get the maximum row to iterate through
    max_row = max(cell["row"] for cell in sheet_data["cells"])

    # Iterate through rows
    for row_idx in range(1, max_row + 1):
        # Get the first cell value in this row (if it exists)
        first_cell_value = cell_map.get((row_idx, 1))
        if first_cell_value is None:
            continue

        first_cell = str(first_cell_value).strip()

        # Check for section headers
        if any(section in first_cell for section in ToggleSection.__members__.values()):
            # If we were processing a previous section, add it to toggles

            if current_section and current_options and current_cell:
                # current_options has values like:
                # [
                #   '{"index":1,"value":"All","cell_row":14,"cell_col":3}',  -> selected option  # noqa: E501
                #   '{"index":1,"value":"All","cell_row":16,"cell_col":3}',
                #   '{"index":2,"value":"Greater than $200,000","cell_row":17,"cell_col":3}',  # noqa: E501
                #   '{"index":3,"value":"Greater than $100,000 up to $200,000","cell_row":18,"cell_col":3}',  # noqa: E501
                #   '{"index":4,"value":"Greater than $0 up to $100,000","cell_row":19,"cell_col":3}'  # noqa: E501
                #  ]
                # so we use the first option as the current chosen value for the toggle.
                # Rest are used as options to expose.

                toggles.append(
                    ToggleInfo(
                        section=current_section,
                        selection_cell=current_cell,
                        options=current_options[1:],
                        selection_cell_row=current_options[0].cell_row,
                        selection_cell_col=current_options[0].cell_col,
                    )
                )

            # Start new section
            current_section = ToggleSection(first_cell)
            current_options = []
            # Selection cell is typically in column C of the next row
            current_cell = f"C{row_idx + 4}"
            continue

        # Process option rows (they start with a number)
        if first_cell and first_cell.isdigit():
            option_index = int(first_cell)
            # Get the option value from column D (index 4)
            option_value = cell_map.get((row_idx, 2))
            option_value = str(option_value).strip()
            if option_value and option_value != "[ ]":
                current_options.append(
                    ToggleOption(
                        index=option_index,
                        value=option_value,
                        cell_row=row_idx + 2,
                        cell_col=3,
                    )
                )

    # Add the last section if exists
    if current_section and current_options and current_cell:  # Add check for current_cell
        toggles.append(
            ToggleInfo(
                section=current_section,
                selection_cell=current_cell,
                selection_cell_row=current_options[0].cell_row,
                selection_cell_col=current_options[0].cell_col,
                options=current_options[1:],
                selected_option_index=current_options[0].index,
            )
        )

    return toggles


def get_toggle_configurations(
    toggle_info: list[ToggleInfo], toggle_values: dict[str, str]
) -> dict[str, ToggleConfiguration]:
    """
    Create toggle configurations with selected values and available options.

    Args:
        toggle_info: List of toggle configurations
        toggle_values: Dictionary of toggle values from user input

    Returns:
        Dictionary mapping section names to their configurations
    """
    configurations = {}

    for toggle in toggle_info:
        section_name = toggle.section.replace(" Toggle", "").lower()
        selected_value = toggle_values.get(
            section_name, toggle.options[0].value if toggle.options else ""
        )

        configurations[section_name] = ToggleConfiguration(
            section=toggle.section,
            selected_value=selected_value,
            available_options=[opt.value for opt in toggle.options],
        )

    return configurations


def get_toggle_edit_operations(
    toggle_info: list[ToggleInfo], toggle_values: dict[str, str]
) -> list[dict[str, Any]]:
    """
    Generate Excel edit operations for each toggle based on selected values.

    Args:
        toggle_info: List of toggle configurations
        toggle_values: Dictionary of toggle values from user input

    Returns:
        List of edit operations with cell locations and values
    """
    edit_operations = []

    for toggle in toggle_info:
        # Get the selected value for this toggle
        section_name = toggle.section.replace(" Toggle", "").lower()
        selected_value = toggle_values.get(section_name)

        # Find the index of the selected value
        selected_index = 1  # Default to first option
        if selected_value:
            for option in toggle.options:
                if option.value.lower() == selected_value.lower():
                    selected_index = option.index
                    break

        # Create edit operation
        edit_operations.append(
            {
                "sheet_name": "Control Panel",
                "cell": toggle.selection_cell,
                "row": toggle.selection_cell_row,
                "column": toggle.selection_cell_col,
                "value": selected_index,
            }
        )

    return edit_operations



from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.folder_models import ListDatasourcesInput, ListDatasourcesOutput

__all__ = ["ListDatasourcesInput", "ListDatasourcesOutput", "list_datasources"]


def list_datasources(
    *, folder_id: int, content_type: str = DEFAULT
) -> ListDatasourcesOutput:
    """
    Lists all datasources in a folder.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ListDatasourcesInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="folder", name="list_datasources", input=input_model.model_dump()
    )
    return ListDatasourcesOutput.model_validate(output_model)



from typing import Any

from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.parallel_thread_process_models import (
    RunTasksInParallelInput,
    RunTasksInParallelOutput,
)

__all__ = ["RunTasksInParallelInput", "RunTasksInParallelOutput", "run_tasks_in_parallel"]


def run_tasks_in_parallel(
    *,
    func_to_run: Any,
    items_to_process: list[Any],
    shared_args: list[Any],
    max_workers: int = DEFAULT,
) -> RunTasksInParallelOutput:
    """Run tasks in parallel.

    Args:
        input: RunTasksInParallelInput containing:
            - func_to_run: The function to execute
            - items_to_process: List of items to process
            - shared_args: Additional arguments to pass to each function call
            - max_workers: Maximum number of parallel workers (default: 5)

    Returns:
        RunTasksInParallelOutput containing the list of results
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = RunTasksInParallelInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="parallel_thread_process",
        name="run_tasks_in_parallel",
        input=input_model.model_dump(),
    )
    return RunTasksInParallelOutput.model_validate(output_model)



from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.outlook_models import (
    EmailItem,
    OutlookInput,
    OutlookOutput,
    ToolSummary,
)

__all__ = [
    "EmailItem",
    "OutlookInput",
    "OutlookOutput",
    "ToolSummary",
    "fetch_outlook_emails",
]


def fetch_outlook_emails(
    *,
    access_token: str,
    folder_name: str = DEFAULT,
    start_date: str | None = DEFAULT,
    end_date: str | None = DEFAULT,
) -> OutlookOutput:
    """
    Fetch emails from a specified Outlook folder, filtered by date and keywords.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = OutlookInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="outlook", name="fetch_outlook_emails", input=input_model.model_dump()
    )
    return OutlookOutput.model_validate(output_model)



from typing import Any

from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.sourcescrub_models import (
    GetCompanyByCompanyIdentifierInput,
    GetCompanyByCompanyIdentifierOutput,
    GetCompanyInfoInput,
    GetCompanyInfoOutput,
    GetMarketIntelligenceInput,
    GetMarketIntelligenceOutput,
    GetSourcesByCompanyIdentifierInput,
    GetSourcesByCompanyIdentifierOutput,
    SearchCompaniesInput,
    SearchCompaniesOutput,
    SearchSourcesInput,
    SearchSourcesOutput,
)

__all__ = [
    "GetCompanyByCompanyIdentifierInput",
    "GetCompanyByCompanyIdentifierOutput",
    "GetCompanyInfoInput",
    "GetCompanyInfoOutput",
    "GetMarketIntelligenceInput",
    "GetMarketIntelligenceOutput",
    "GetSourcesByCompanyIdentifierInput",
    "GetSourcesByCompanyIdentifierOutput",
    "SearchCompaniesInput",
    "SearchCompaniesOutput",
    "SearchSourcesInput",
    "SearchSourcesOutput",
    "get_company_info",
    "search_companies",
    "get_market_intelligence",
    "get_company_by_company_identifier",
    "get_sources_by_company_identifier",
    "search_sources",
]


def get_company_info(
    *, domain: str, data: dict[str, Any] = DEFAULT, timeout: float = DEFAULT
) -> GetCompanyInfoOutput:
    """Get detailed information about a specific company by domain."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetCompanyInfoInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="sourcescrub", name="get_company_info", input=input_model.model_dump()
    )
    return GetCompanyInfoOutput.model_validate(output_model)


def search_companies(
    *, data: dict[str, Any] = DEFAULT, timeout: float = DEFAULT
) -> SearchCompaniesOutput:
    """Search for companies using SourceScrub API."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = SearchCompaniesInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="sourcescrub", name="search_companies", input=input_model.model_dump()
    )
    return SearchCompaniesOutput.model_validate(output_model)


def get_market_intelligence(
    *, company_identifier: str = DEFAULT, timeout: float = DEFAULT
) -> GetMarketIntelligenceOutput:
    """Search market intelligence by company identifier using SourceScrub API."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetMarketIntelligenceInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="sourcescrub",
        name="get_market_intelligence",
        input=input_model.model_dump(),
    )
    return GetMarketIntelligenceOutput.model_validate(output_model)


def get_company_by_company_identifier(
    *, company_identifier: str = DEFAULT, timeout: float = DEFAULT
) -> GetCompanyByCompanyIdentifierOutput:
    """Get company by company identifier using SourceScrub API."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetCompanyByCompanyIdentifierInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="sourcescrub",
        name="get_company_by_company_identifier",
        input=input_model.model_dump(),
    )
    return GetCompanyByCompanyIdentifierOutput.model_validate(output_model)


def get_sources_by_company_identifier(
    *, company_identifier: str = DEFAULT, timeout: float = DEFAULT
) -> GetSourcesByCompanyIdentifierOutput:
    """Get sources by company identifier using SourceScrub API."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetSourcesByCompanyIdentifierInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="sourcescrub",
        name="get_sources_by_company_identifier",
        input=input_model.model_dump(),
    )
    return GetSourcesByCompanyIdentifierOutput.model_validate(output_model)


def search_sources(
    *, data: dict[str, Any] = DEFAULT, timeout: float = DEFAULT
) -> SearchSourcesOutput:
    """Search sources using SourceScrub API."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = SearchSourcesInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="sourcescrub", name="search_sources", input=input_model.model_dump()
    )
    return SearchSourcesOutput.model_validate(output_model)



from typing import Any

from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.call_task_output_parser_models import (
    CallTaskOutputParserInput,
    CallTaskOutputParserOutput,
)

__all__ = [
    "CallTaskOutputParserInput",
    "CallTaskOutputParserOutput",
    "parse_call_task_output",
]


def parse_call_task_output(
    *, transcript: str = DEFAULT, output_schema: dict[str, Any] = DEFAULT
) -> CallTaskOutputParserOutput:
    """
    Parse a call transcript using LLM to extract structured output according to a schema.

    This tool analyzes call transcripts and extracts information that matches the
    provided output schema, which is typically from a CallTaskDefinition.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = CallTaskOutputParserInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="call_task_output_parser",
        name="parse_call_task_output",
        input=input_model.model_dump(),
    )
    return CallTaskOutputParserOutput.model_validate(output_model)



from typing import Any, Literal

from a79.client import A79Client
from a79.models.tools import DEFAULT
from a79.models.tools.snowflake_models import (
    CSVDataResponse,
    ExecuteQueryInput,
    ExecuteQueryOutput,
    ReadTableInput,
    ReadTableOutput,
    SnowflakeConfig,
    TableInfo,
    UploadDataInput,
    UploadDataOutput,
)

__all__ = [
    "CSVDataResponse",
    "ExecuteQueryInput",
    "ExecuteQueryOutput",
    "ReadTableInput",
    "ReadTableOutput",
    "SnowflakeConfig",
    "TableInfo",
    "UploadDataInput",
    "UploadDataOutput",
    "upload_data",
    "read_table",
    "execute_query",
]


def upload_data(
    *,
    config: SnowflakeConfig = DEFAULT,
    dataframe: Any,
    table_name: str,
    if_exists: Literal["fail", "replace", "append"] = DEFAULT,
) -> UploadDataOutput:
    """
    Uploads a single dataframe to a Snowflake table.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = UploadDataInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="snowflake", name="upload_data", input=input_model.model_dump()
    )
    return UploadDataOutput.model_validate(output_model)


def read_table(
    *,
    config: SnowflakeConfig,
    table_name: str,
    database: str | None = DEFAULT,
    schema_name: str | None = DEFAULT,
    limit: int | None = DEFAULT,
) -> ReadTableOutput:
    """Reads data from a Snowflake table and returns it as a paginated response."""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ReadTableInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="snowflake", name="read_table", input=input_model.model_dump()
    )
    return ReadTableOutput.model_validate(output_model)


def execute_query(
    *, config: SnowflakeConfig, query: str, page: int = DEFAULT, page_size: int = DEFAULT
) -> ExecuteQueryOutput:
    """
    Executes a SQL query against Snowflake and returns paginated results.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ExecuteQueryInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="snowflake", name="execute_query", input=input_model.model_dump()
    )
    return ExecuteQueryOutput.model_validate(output_model)



from pydantic import BaseModel, Field

from common_py.model.enums import ContentType

from . import ToolOutput


class ParseDocumentInput(BaseModel):
    document_input: str = Field(
        default="",
        json_schema_extra={"mandatory": True},
        description=(
            "URL of the document to download and parse, or the document content itself"
        ),
    )
    content_type: ContentType = Field(
        default=ContentType.PLAIN_TEXT,
        description=(
            "Content type of the document "
            "(auto-detected for URLs, optional for direct content)"
        ),
    )


class ParseDocumentOutput(ToolOutput):
    parsed_text: str = Field(
        default="", description="The parsed text content from the document"
    )



"""A79 SDK tools module."""

from . import (
    audio,
    call_task_generator,
    call_task_output_parser,
    clinical_trials_search,
    datasource,
    document,
    edgartool,
    email,
    excel,
    firecrawl,
    folder,
    google,
    llm,
    nl_to_sql,
    operators,
    outlook,
    parallel_thread_process,
    perplexity,
    pii,
    pitchbook,
    podcast,
    project,
    proquest,
    pubmed,
    react_agent,
    snowflake,
    sourcescrub,
    thoughtspot,
    workflow,
    worksheet,
)

__all__ = [
    "audio",
    "call_task_generator",
    "call_task_output_parser",
    "clinical_trials_search",
    "datasource",
    "document",
    "edgartool",
    "email",
    "excel",
    "firecrawl",
    "folder",
    "google",
    "llm",
    "nl_to_sql",
    "operators",
    "outlook",
    "parallel_thread_process",
    "perplexity",
    "pii",
    "pitchbook",
    "podcast",
    "project",
    "proquest",
    "pubmed",
    "react_agent",
    "snowflake",
    "sourcescrub",
    "thoughtspot",
    "workflow",
    "worksheet",
]



from pydantic import BaseModel

from common_py.connector.tap_pubmed.models import ArticleRecord

from . import ToolOutput


class GetArticlesInput(BaseModel):
    feed_url: str
    limit: int = 15


class GetArticlesOutput(ToolOutput):
    articles: list[ArticleRecord]



from pydantic import BaseModel, Field

from common_py.model import enums

from . import ToolOutput


class ListDatasourcesInput(BaseModel):
    folder_id: int = Field(
        description="The ID of the folder to read datasources from",
        json_schema_extra={"field_type": enums.CustomDataType.FOLDER.value},
    )
    content_type: str = Field(default="", description="Filter by content type")


class ListDatasourcesOutput(ToolOutput):
    datasource_ids: list[int] = Field(default_factory=list)



import typing as t

from common_py.workflow.nodes.node import NodeOutputData
from common_py.workflow.nodes.tools.human_feedback import (
    HumanFeedbackTool as HumanFeedbackInternal,
)
from common_py.workflow.nodes.tools.human_feedback import HumanFeedbackToolInput


def HumanFeedbackTool(**kwargs: t.Any) -> NodeOutputData:
    """
    This tool reads content from a worksheet by ID.
    """
    input = HumanFeedbackToolInput(**kwargs)
    tool = HumanFeedbackInternal(node_input_data=input)
    return tool.execute()
